@@ -5,7 +5,7 @@ import os
 import click
 import yaml
 
-from . import constants
+from servicecatalog_puppet import constants
 from betterboto import client as betterboto_client
 
 logger = logging.getLogger(__file__)
@@ -81,7 +81,6 @@ def get_stack_output_for(cloudformation, stack_name):
 def get_default_parameters_for_stack(cloudformation, stack_name):
     logger.info(f"Getting default parameters for for {stack_name}")
     existing_stack_params_dict = {}
-    #errored
     summary_response = cloudformation.get_template_summary(
         StackName=stack_name,
     )
@@ -145,6 +144,10 @@ def provision_product_with_plan(
         ProvisioningArtifactId=provisioning_artifact_id,
         ProvisioningParameters=provisioning_parameters,
         Tags=[
+            {
+                'Key': 'ServiceCatalogPuppet:Actor',
+                'Value': "Generated",
+            },
             {
                 'Key': 'launch_name',
                 'Value': launch_name,
@@ -177,10 +180,11 @@ def provision_product_with_plan(
             f"changes: {yaml.safe_dump(response.get('ResourceChanges'))}"
         )
         if len(response.get('ResourceChanges')) == 0:
-            logger.warning(f"{uid} :: There are no resource changes in this plan, "
-                        f"running this anyway - your product will be marked as tainted as your CloudFormation changeset"
-                        f"will fail but your product will be the correct version and in tact.")
-
+            logger.warning(
+                f"{uid} :: There are no resource changes in this plan, "
+                f"running this anyway - your product will be marked as tainted as your CloudFormation changeset"
+                f"will fail but your product will be the correct version and in tact."
+            )
 
         logger.info(f"{uid} :: executing changes")
         service_catalog.execute_provisioned_product_plan(PlanId=plan_id)
@@ -210,7 +214,7 @@ def provision_product_with_plan(
                 execute_status = provisioned_product_detail.get('Status')
                 if execute_status in ['AVAILABLE', 'TAINTED', 'EXECUTE_SUCCESS']:
                     break
-                elif execute_status ==  'ERROR':
+                elif execute_status == 'ERROR':
                     raise Exception(f"{uid} :: Execute failed: {execute_status}: {provisioned_product_detail.get('StatusMessage')}")
                 else:
                     time.sleep(5)
@@ -252,6 +256,10 @@ def provision_product(
         ProvisionedProductName=launch_name,
         ProvisioningParameters=provisioning_parameters,
         Tags=[
+            {
+                'Key': 'ServiceCatalogPuppet:Actor',
+                'Value': "Generated",
+            },
             {
                 'Key': 'launch_name',
                 'Value': launch_name,
@@ -279,7 +287,7 @@ def provision_product(
         execute_status = provisioned_product_detail.get('Status')
         if execute_status in ['AVAILABLE', 'TAINTED', 'EXECUTE_SUCCESS']:
             break
-        elif execute_status ==  'ERROR':
+        elif execute_status == 'ERROR':
             raise Exception(f"{uid} :: Execute failed: {execute_status}: {provisioned_product_detail.get('StatusMessage')}")
         else:
             time.sleep(5)
@@ -326,7 +334,7 @@ def update_provisioned_product(
         execute_status = provisioned_product_detail.get('Status')
         if execute_status in ['AVAILABLE', 'TAINTED', 'EXECUTE_SUCCESS']:
             break
-        elif execute_status ==  'ERROR':
+        elif execute_status == 'ERROR':
             raise Exception(f"{uid} :: Execute failed: {execute_status}: {provisioned_product_detail.get('StatusMessage')}")
         else:
             time.sleep(5)
